# Generated by Django 3.2.8 on 2021-11-13 23:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('accounts', '0003_auto_20211028_0404'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='username',
            field=models.CharField(default='username', max_length=100, unique=True),
            preserve_default=False,
        ),
    ]



from django.db import models
from django.db.models.signals import post_save
from django.dispatch import receiver

from django.contrib.auth.hashers import make_password
from django.contrib.auth.models import AbstractBaseUser, BaseUserManager, PermissionsMixin
import random

from django.template.loader import render_to_string
from django.core.mail import EmailMultiAlternatives

from django.conf import settings


class CustomManager(BaseUserManager):
    def create_user(self, email, username, password, **extra_fields):
        if not email:
            raise ValueError('email address is required')
        
        if not username:
            raise ValueError('user must have an username')
        
        email = self.normalize_email(email)
        user = self.model(email=email, username=username, **extra_fields)
        user.set_password(make_password(password))
        user.save(using=self._db)
        return user
    
    def create_superuser(self, email, username, password, **extra_fields):
        extra_fields.setdefault('is_staff', True)
        extra_fields.setdefault('is_superuser', True)
        extra_fields.setdefault('is_active', True)
        extra_fields.setdefault('is_admin', True)

        if extra_fields.get('is_staff') is not True:
            raise ValueError("Superuser must be have is_staff True")

        if extra_fields.get('is_superuser') is not True:
            raise ValueError("Superuser must be have is_superuser True")

        if extra_fields.get('is_active') is not True:
            raise ValueError("Superuser must be have is_active True")
        
        if extra_fields.get('is_admin') is not True:
            raise ValueError("Superuser must be have is_admin True")
        
        user = self.create_user(email, username, password, **extra_fields)

        return user

    


class User(AbstractBaseUser, PermissionsMixin):
    USER_TYPE = (
        ("visitor", "visitor"),
        ("developer", "developer"),
    )
    email = models.EmailField(unique=True)
    username = models.CharField(max_length=100, unique=True)
    REQUIRED_FIELDS = ['username']
    USERNAME_FIELD = 'email'
    user_type = models.CharField(max_length=100, choices=USER_TYPE, default=USER_TYPE[0])
    is_staff = models.BooleanField(default=False)
    is_active = models.BooleanField(default=False)
    is_verify = models.BooleanField(default=False)
    is_superuser = models.BooleanField(default=False)
    is_admin = models.BooleanField(default=False)
    objects = CustomManager()

    def __str__(self) -> str:
        return str(self.email)


class Verification(models.Model):
    user = models.OneToOneField(User, on_delete=models.CASCADE, related_name='verification')
    account_code = models.CharField(max_length=20, blank=True, null=True)
    verify_code = models.CharField(max_length=10)

    def __str__(self):
        return str(self.user.email)
    
    @receiver(post_save, sender=User)
    def create_verification(sender, instance, created, **kwargs):
        if created:
            current_user = Verification.objects.create(user=instance)

            # sending email 
            ac_code = ''.join([str(random.randint(0, 999)).zfill(3) for _ in range(4)])
            current_user.account_code = ac_code
            # send  email
            user_email = current_user.user.email
            get_username = current_user.user.username
            # end split
            code_num = ''.join([str(random.randint(0, 999)).zfill(3) for _ in range(2)])
            # save random number to database
            current_user.verify_code = code_num
            # end saving
            subject = "Verify your email address"
            # html_message = render_to_string('accounts/email.html')
            email_from = settings.EMAIL_HOST_USER
            recipient_list = [user_email]
            # send_mail( subject, html_message, email_from, recipient_list )
            sitename = SaveDomain.objects.all().order_by('-id').first()
            context = ({
                'username': get_username,
                'code': code_num,
                'domain': sitename.get_domain()
            })
            text_content = render_to_string('account/email.txt', context)
            html_content = render_to_string('account/email.html', context)
            email = EmailMultiAlternatives(subject=subject, body=text_content, from_email=email_from, to=recipient_list, reply_to=[email_from,])
            email.attach_alternative(html_content, "text/html")
            email.send(fail_silently=False)
            # end here

    @receiver(post_save, sender=User)
    def save_verification(sender, instance, **kwargs):
        instance.verification.save()
        



    

class SaveDomain(models.Model):
    domain = models.CharField(max_length=255)

    def get_domain(self, *args, **kwargs):
        if self.domain:
            return self.domain
        else:
            return None


# Generated by Django 3.1.6 on 2021-10-27 04:27

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('accounts', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelManagers(
            name='user',
            managers=[
            ],
        ),
    ]



from django.shortcuts import render, redirect
from django.urls import reverse
from django.http import HttpResponseRedirect
from accounts.forms import RegisterForm
from django.contrib.auth import authenticate, login
from django.views.generic import TemplateView
from accounts.models import SaveDomain
from django.contrib.auth import get_user_model
User = get_user_model()

from django.conf import settings

class RegisterTemplateView(TemplateView):
    def get(self, request, *args, **kwargs):
        # save the domain name from the request
        domains = SaveDomain.objects.create(domain=str(request.get_host()))

        # let check the current user is authenticated or not
        if request.user.is_authenticated:
            return redirect('/')
        else:
            form = RegisterForm()
            context = {
                'form': form
            }
            return render(request, 'account/register-new.html', context)
    def post(self, request, *args, **kwargs):
        if request.user.is_authenticated:
            return redirect(settings.LOGIN_REDIRECT_PATH)
        else:
            form = RegisterForm()
            if request.method == 'post' or request.method == 'POST':
                form = RegisterForm(request.POST)
                if form.is_valid():
                    try:
                        user = form.save()
                        get_user_verification_obj = user.verification
                        acc_code = get_user_verification_obj.account_code
                        return redirect(reverse('accounts:verify', kwargs={'account_code': acc_code}))
                    except:
                        return HttpResponseRedirect(request.META.get('HTTP_REFERER'))

            context = {'form': form}
            return render(request, 'account/register-new.html', context)


def account_verification(request, account_code):
    try:
        if request.user.is_authenticated:
            return redirect('accounts:register')
        else:
            if account_code:
                get_user = User.objects.get(verification__account_code=account_code)
                get_code = get_user.verification.verify_code
                if request.method == 'post' or request.method == 'POST':
                    check_code = request.POST.get('code')
                    if get_code == check_code:
                        get_user.is_active = True
                        get_user.is_verify = True
                        get_user.save()
                        return redirect('accounts:login')
                    else:
                        return HttpResponseRedirect(request.META.get('HTTP_REFERER'))

                context = {}
                return render(request, 'account/verification-new.html', context)
            else:
                return redirect('accounts:register')
    except:
        return render(request, 'account/verification-new.html')



class LoginTemplateView(TemplateView):
    def get(self, request, *args, **kwargs):
        if request.user.is_authenticated:
            return redirect(settings.LOGIN_REDIRECT_PATH)
        else:
            return render(request, 'account/login-new.html')

    def post(self, request, *args, **kwargs):
        if request.method == 'post' or request.method == 'POST':
            username = request.POST.get('username')
            password = request.POST.get('password')
            user = authenticate(request, username=username, password=password)
            if user is not None:
                login(request, user)
                return redirect(settings.LOGIN_REDIRECT_PATH)
        return render(request, 'account/login-new.html')






from django.contrib import admin
from django.contrib.auth import models

from accounts.models import User, Verification
from accounts.forms import UserCreationForm


class UserAdmin(admin.ModelAdmin):
    list_display = ('id', 'username', 'email', 'is_active', 'is_admin')



class VerifyAdmin(admin.ModelAdmin):
    list_display = ('id', 'user', 'account_code', 'verify_code')


admin.site.register(User, UserAdmin)
admin.site.register(Verification, VerifyAdmin)



from django import forms
from accounts.models import User
from django.contrib.auth.forms import UserCreationForm


class RegisterForm(UserCreationForm):
    class Meta:
        model = User
        fields = ['username', 'email', 'password1', 'password2']

        widgets = {
            'username': forms.TextInput(attrs={'class': 'form-control'}),
            'email': forms.TextInput(attrs={'class': 'form-control'}),
            'password1': forms.TextInput(attrs={'class': 'form-control'}),
            'password2': forms.TextInput(attrs={'class': 'form-control'}),
            
        }

        def clean_email(self):
            email = self.cleaned_data["email"].lower()
            try:
                user = User.objects.get(email=email)
            except Exception as e:
                return email
            raise forms.ValueError(f'email {self.email} is Already used.')
        
        def clean_username(self):
            username = self.cleaned_data["username"]
            try:
                user = User.objects.get(username=username)
            except Exception as e:
                return username
            raise forms.ValueError(f'email {self.username} is Already used.')
    


from django.contrib.auth.backends import ModelBackend
from django.contrib.auth import get_user_model
from django.db.models import Q

User = get_user_model()

class EmailOrUsernameBackend(ModelBackend):
    def authenticate(self, request, username=None, password=None, **kwargs):
        try:
            user = User.objects.get(Q(username__iexact=username) | Q(email__iexact=username))
        except User.DoesNotExist:
            User().set_password(password)
            return
        except User.MultipleObjectsReturned:
            user = User.objects.filter(Q(username__iexact=username) | Q(email__iexact=username)).order_by('id').first()
        if user.check_password(password) and self.user_can_authenticate(user):
            return user 


from os import name
from django.urls import path
from accounts import views

app_name = 'accounts'

urlpatterns = [
    path('register/', views.RegisterTemplateView.as_view(), name='register'),
    path('verification/<str:account_code>/', views.account_verification, name='verify'),
    path('login/', views.LoginTemplateView.as_view(), name='login'),

]



class AccountsRouter:
    route_app_labels = {'accounts', 'blog', 'dashboard', 'store', 'order', 'vendor_store', 'admin', 'contenttypes', 'sessions', 'auth', }

    def db_for_read(self, model, **hints):
        if model._meta.app_label in self.route_app_labels:
            return 'accounts'
        return None

    def db_for_write(self, model, **hints):
        if model._meta.app_label in self.route_app_labels:
            return 'accounts'
        return None

    def allow_relation(self, obj1, obj2, **hints):
        if (
            obj1._meta.app_label in self.route_app_labels or 
            obj2._meta.app_label in self.route_app_labels
        ):
            return True
        return None

    def allow_migrate(self, db, app_label, model_name=None, **hints):
        if app_label in self.route_app_labels:
            return db == 'accounts'
        return None
